% Evaluating Factor Pricing Models Using High Frequency Panels
% by Yoosoon Chang, Yongok Choi, Hagen Kim, Joon Park
%
% alphas of Industry portfolios: CAPM, Random Time
% Input: 1. FF_Factors_daily.txt
%           1963.7.1 ~ 2009. 3.31 for the risk free rate
%        2. 30_Industry_portfolios_Daily_VW.txt
%           1963.7.1 ~ 2008. 12.31 for the 30 Industry portfolios
%        3. Dm_Mkt.txt
%           1963.7.2 ~ 2008. 12.31 for the demeaned excess market return
% Output: alpha_ind_2.eps
%
% 2014-03-05  
% Version 2.0 
% By Yongok Choi (choiyongok@gmail.com)

clear all; close all; clc;
addpath('..\..\SharedPrg');

% Loading data sets
% 1963.7.1 ~ 2008. 12.31 for the 30 portfolios
% 1963.7.1 ~ 2009. 3.31 for the F-F factor
% 1963.7.2 ~ 2008. 12.31 for the demeaned excess Market return

load ('..\..\Data\30_Industry_portfolios_Daily_VW.txt'); %Value-weighted returns
load ('..\..\Data\FF_Factors_daily.txt'); % Fama-French factors and risk-free rate
load ('..\..\Data\Dm_Mkt.txt');                    % Demeaned excess market return

% Define variables
% 30 returns, percentage terms
R_vw    = X30_Industry_portfolios_Daily_VW(1:11455,2:end); %Value-weighted returns
[m,n] = size(R_vw); % m: # of rows, n: # of columns

% FF factors and risk-free rate
Mkt  = FF_Factors_daily(1:m,2);             %Market minus risk free rate
Rf   = FF_Factors_daily(1:m,end);           %Risk free rate

% Excess 1-peiod holding period returns and factors in percentage terms
hprx_vw = R_vw(2:end,:)-kron(Rf(1:end-1,:),ones(1,n));
Mkt     = Mkt(2:end,:);
Dm_Mkt  = Dm_Mkt(1:end,:);

Date    = X30_Industry_portfolios_Daily_VW(2:m,1);      %Date
Year    = floor(Date/10000);                            %Year
Month   = floor(Date/100)-Year*100;                     %Month
Mindex  = (Year-1963)*12+Month;                         %Monthly Index

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step0: Set the period that we want to investigate
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SY      = 1963;             %Start Year
SM      = 7;                %Start Month
EY      = 2008;             %End Year
EM      = 12;               %End Month

DI_1    = Mindex >= (SY-1963)*12+SM;
DI_2    = Mindex <= (EY-1963)*12+EM;
DI      = DI_1 & DI_2;

hprx_vw     = hprx_vw(DI,:);
Mkt         = Mkt(DI,:);
Dm_Mkt      = Dm_Mkt(DI,:);

Date        = Date(DI,:);
Year        = Year(DI,:);
Month       = Month(DI,:); 
Mindex      = Mindex(DI,:);

MSE_p       = 0.05;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step1: Fix at monthly average QV of excess mkt return for time change
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

QV      = cumsum(Dm_Mkt.^2,1);                     %Quadratic Variation
TQV     = sum(Dm_Mkt.^2,1);                        %Total Quadratic Variation
MDelta  = TQV/(max(Mindex)-min(Mindex)+1);      %Monthly Equivalent Delta

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step2: Jump Test by Lee and Mykland
%       - get the date and month information
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

K       = 16;                               %window size
alpha   = 0.05;                             %significance level

rejidx   = JT_LM(Dm_Mkt,K,alpha);              %reject index(jump indicate)
% Jumpdate = Date(rejidx);                  %Date for Jumps

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step3: Select valid time changed sample
%       - remove samples whose period includes Jumps
%       - remove samples whose contribution to MSE is high
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[TCD,Tms,DeltaV] = TimeChange(Dm_Mkt,MDelta,3);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Constructing Time Change Interval Information
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Sdate   = [Date(1);Date(Tms(1:end-1)+1)];
Edate   = Date(Tms(1:end));
Nobs    = [Tms(1);Tms(2:end)-Tms(1:end-1)];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% treat Jumps
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Jump    = zeros(length(Tms),1);
for i = 1:1:length(Tms)
    if i == 1
        Jump(i) = any(rejidx(1:Tms(1)));
    else
        Jump(i) = any(rejidx(Tms(i-1)+1:Tms(i)));
    end
end

VP_Jump = Jump < 1; % Valid Periods without Jumps
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% treat MSE
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
S_SE    = sort((DeltaV-MDelta).^2,1,'descend');%Sorted Square Error
VP_MSE  = (DeltaV-MDelta).^2 < S_SE(round(length(Tms)*MSE_p));   %Valid Periods in terms of MSE 
VP      = VP_MSE & VP_Jump;

fprintf('Data period: from %6.0f - %2.0f to %6.0f - %2.0f',SY,SM,EY,EM);
fprintf('\n The number of months is                             %10.4f. ', max(Mindex)-min(Mindex)+1);
fprintf('\n The number of the time changed samples is           %10.4f', length(Nobs));
fprintf('\n The number of samples with Jumps is                 %10.4f. ', sum(Jump));
fprintf('\n The number of samples violating MSE criterion is    %10.4f. ', round(length(Tms)*MSE_p));
fprintf('\n The number of used samples is                       %10.4f. ', sum(VP));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step4: Build Monthly Data 
%       - remove months which jumps happened
%       - remove months which violate MSE criterion
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

NM      = length(Nobs);
MY      = zeros(NM,n);
MX      = zeros(NM,2);

for i=1:1:NM;
    S_I_1   = Date >= Sdate(i,1);
    S_I_2   = Date <= Edate(i,1);
    S_I     = S_I_1 & S_I_2;
    Temp_Y  = hprx_vw(S_I,:);
    Temp_X  = Mkt(S_I,:);
    MX(i,:) = [sum(S_I,1)*100/252 sum(Temp_X,1)];
    MY(i,:) = sum(Temp_Y,1);
end

MX  = MX(VP,:);
MY  = MY(VP,:);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step5: For each i=1, ... , 25, estimate betas by running OLS over Monthly
% data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

BigBhat = inv(MX'*MX)*MX'*MY;
BigBhatT = BigBhat';
BetaRst_1=[ BigBhatT(:,1);0;0;mean(BigBhatT(:,1));std(BigBhatT(:,1)) ];
BetaRst_2=[ BigBhatT(:,2);0;0;mean(BigBhatT(:,2));std(BigBhatT(:,2)) ];
BetaRst = [BetaRst_1 BetaRst_2];
Uhat    = MY - MX*BigBhat;
 
[nr,nc]     = size(MX);
s2          = sum(Uhat.^2,1)/(nr-nc);
Bigse       = sqrt(diag(inv(MX'*MX))*s2);
BigseT      = Bigse';

BigEstRst   = zeros(n,nc*2);
for i=1:1:nc;
    BigEstRst(:,2*(i-1)+1) = BigBhatT(:,i);
    BigEstRst(:,2*(i-1)+2) = BigseT(:,i);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step6: construct fitted residuals at daily frequncy using beta hat 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

BigUhat = hprx_vw - [ones(length(Mkt),1)*100/252 Mkt]*BigBhat;

% RV(R_m-R_f) = b_i *RV(resid. of R_i-R_f) + error_i, for i=1,,25
% and plot RV(R_m-R_f) and hat(b)_i *RV(R_i-R_f) for i=1,,25.  

RV_resid_i = zeros(length(Date),n);

for i= 1:1:n;
    Temp_X = cumsum(BigUhat(:,i).^2,1);
    Temp_Y = cumsum(Mkt.^2,1);
    RV_resid_i(:,i)= inv(Temp_X'*Temp_X)*Temp_X'*Temp_Y*Temp_X;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step4: Plot alpha_i vs index 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Group1 = [BigBhat(1,1:3)';BigBhat(1,5:7)';BigBhat(1,10);BigBhat(1,26:28)'];
Group2 = [BigBhat(1,9);BigBhat(1,12:16)';BigBhat(1,18:20)';BigBhat(1,24)];
Group3 = [BigBhat(1,21:23)'];
Group4 = BigBhat(1,8);
Group5 = [BigBhat(1,4);BigBhat(1,11);BigBhat(1,15);BigBhat(1,17:18)';BigBhat(1,22);BigBhat(1,25);BigBhat(1,29:30)'];

figure1 = figure;
set(figure1, 'Units', 'inches');
set(figure1, 'Position', [1 2 2.9*3 2*3]);
set(figure1, 'PaperPositionMode', 'manual');
set(figure1, 'PaperUnits', 'inches');
set(figure1, 'PaperPosition', [1 1 2.9*3 2*3]);

axes1 = axes('Parent',figure1,'XTick',[1 2 3 4 5 6 7 8 9 10],'LineWidth',2,'Layer','top','FontSize',16,...
    'FontName','Tahoma');
box(axes1,'on');
hold(axes1,'all');
xlim(axes1,[1 10]);

plot([1 2 3 4 5 6 7 8 9 10],Group1,'o-',...
     [1 2 3 4 5 6 7 8 9 10],Group2,'-^',...
     [1 2 3], Group3, '-*', ...
     [1], Group4,'-s', ...
     [1 2 3 4 5 6 7 8 9],Group5,'-d','linewidth',1.5);
 
title('Random Time','FontSize',20,'FontName','Tahoma');  

legend('Cnsmr','Manuf','HiTec','Hlth','Other');
set(legend,'Location','SouthEast');
gridxy([],0,'Linestyle',':','linewidth',2) ;    
print(figure1,'-depsc','alpha_ind_2.eps')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step8: Estimate error variance matrix
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

MU      = zeros(NM,n);
Omegahats = zeros(n,n,NM);

for i=1:1:NM;
    S_I_1   = Date >= Sdate(i,1);
    S_I_2   = Date <= Edate(i,1);
    S_I     = S_I_1 & S_I_2;
    Temp_U  = BigUhat(S_I,:);
    Temp_Omega = zeros(n,n);
    [nrow,ncol] = size(Temp_U);
    for j=1:1:nrow;
        Temp_Omega = Temp_Omega + Temp_U(j,:)'*Temp_U(j,:);
    end;
    Omegahats(:,:,i) = Temp_Omega;
end
Sigmahat2 = Omegahats(:,:,VP);
Sigmahat = mean(Sigmahat2,3);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step9: Calcuate Wald statistic
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

X1 = MX(:,1);
X2 = MX(:,2);

star = inv(X1'*X1-X1'*X2*inv(X2'*X2)*X2'*X1);
Wald = inv(star)*BigBhat(1,:)*inv(Sigmahat)*BigBhat(1,:)';

fprintf('\n H_0: alpha_i = 0 for all i.');
fprintf('\n Wald statistic is                                   %10.4f. ', Wald);
fprintf('\n p-value from Wald statistic is                      %10.4f. \n',1-chi2cdf(Wald,n));       

