% Evaluating Factor Pricing Models Using High Frequency Panels
% by Yoosoon Chang, Yongok Choi, Hagen Kim, Joon Park
%
% Fama-French Alphas and Betas of (Size, B/M) Portfolios, Fixed Time
% Input: 1. FF_Factors_daily.txt
%           1963.7.1 ~ 2009. 3.31 for the risk free rate
%        2. VW_25ptfo_daily.txt
%           1963.7.1 ~ 2008. 12.31 for the 25 portfolios(Size,B/M)
%        3. Dm_Mkt.txt
%           1963.7.2 ~ 2008. 12.31 for the demeaned excess market return
% Output: Two Figures: alpha_FF_1.eps and beta_FF_1.eps
%
% 2014-03-05  
% Version 2.0 
% By Yongok Choi (choiyongok@gmail.com)

clear all; close all; clc;
addpath('..\..\SharedPrg');

% Loading data sets
% 1963.7.1 ~ 2008. 12.31 for the 5x5 portfolio
% 1963.7.1 ~ 2009. 3.31 for the F-F factor
% 1963.7.2 ~ 2008. 12.31 for the demeaned excess market return

load ('..\..\Data\VW_25ptfo_daily.txt');           %Value-weighted returns
load ('..\..\Data\FF_Factors_daily.txt');          % Fama-French factors and risk-free rate
load ('..\..\Data\Dm_Mkt.txt');                    % Demeaned excess market return

% Define variables
% 25 returns, percentage terms
% Size = 1 (smallest) to 5 (largest) scales, B/M = 1 (low) to 5 (high) 
% R(i), i = 1,...,25. i=1 corresponds to (size,B/M)=(1,1), i=2,
% (1,2),..., i=5 to (1,5). Then, i=6 is (2,1) and so on. Thus, i=25 means
% (biggest firm, highest B/M). 

R_vw    = VW_25ptfo_daily(:,2:end); %Value-weighted returns
[m,n]   = size(R_vw); % m: # of rows, n: # of columns

% FF factors and risk-free rate
Mkt  = FF_Factors_daily(1:m,2);     %Market minus risk free rate
Rf   = FF_Factors_daily(1:m,end);   %Risk free rate
SMB  = FF_Factors_daily(1:m,3);     %Small minus Big ME
HML  = FF_Factors_daily(1:m,4);     %High minus Low BE/ME

% Excess 1-peiod holding period returns and factors in percentage terms
hprx_vw = R_vw(2:end,:)-kron(Rf(1:end-1,:),ones(1,n));
Mkt     = Mkt(2:end,:);
SMB     = SMB(2:end,:);
HML     = HML(2:end,:);
Dm_Mkt  = Dm_Mkt(1:end,:);

Date    = VW_25ptfo_daily(2:end,1);     %Date
Year    = floor(Date/10000);            %Year
Month   = floor(Date/100)-Year*100;     %Month
Mindex  = (Year-1963)*12+Month;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step0: Set the decade that we want to investigate
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SY      = 1963;
SM      = 7;
EY      = 2008;
EM      = 12;

DI_1    = Mindex >= (SY-1963)*12+SM;
DI_2    = Mindex <= (EY-1963)*12+EM;
DI      = DI_1 & DI_2;

hprx_vw     = hprx_vw(DI,:);
Mkt         = Mkt(DI,:);
SMB         = SMB(DI,:);
HML         = HML(DI,:);
Dm_Mkt      = Dm_Mkt(DI,:);

Date        = Date(DI,:);
Year        = Year(DI,:);
Month       = Month(DI,:); 
Mindex      = Mindex(DI,:);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step1: Jump Test by Lee and Mykland
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

K       = 16;                               %window size
alpha   = 0.05;                             %significance level

rejidx   = JT_LM(Dm_Mkt,K,alpha);              %reject index(jump indicate)
% Jumpdate = Date(rejidx);                  %Date for Jumps

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step2: Build Monthly Data 
%       - remove months which jumps happened
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SMI     = (SY-1963)*12+SM;
EMI     = (EY-1963)*12+EM;
NM      = EMI-SMI+1;
MY      = zeros(NM,n);
MX      = zeros(NM,4);
MU      = zeros(NM,n);
VP      = true(NM,1);
for i=1:1:NM;
    S_I     = Mindex == i+SMI-1;
    Temp_Y  = hprx_vw(S_I,:);
    Temp_X  = [Mkt(S_I,:) SMB(S_I,:) HML(S_I,:)];
    MX(i,:) = [sum(S_I,1)/252*100 sum(Temp_X,1)];
    MY(i,:) = sum(Temp_Y,1);
    VP(i)   = sum(rejidx(S_I),1) == 0 ;    
end

MX  = MX(VP,:);
MY  = MY(VP,:);

fprintf('Data period: from %6.0f - %2.0f to %6.0f - %2.0f',SY,SM,EY,EM);
fprintf('\n the number of the months is         %10.4f. ', NM);
fprintf('\n The number of samples with Jumps is %10.4f. ', NM-sum(VP));
fprintf('\n The number of used samples is       %10.4f. ', sum(VP));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step3: For each i=1, ... , 25, estimate betas by running OLS over Monthly
% data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

BigBhat = inv(MX'*MX)*MX'*MY;
BigBhatT = BigBhat';
BetaRst = [ BigBhatT;mean(BigBhatT,1);std(BigBhatT,1) ];
Uhat    = MY - MX*BigBhat;

[nr,nc]     = size(MX);
s2          = sum(Uhat.^2,1)/(nr-nc);
Bigse       = sqrt(diag(inv(MX'*MX))*s2);
BigseT      = Bigse';

BigEstRst   = zeros(n,nc*2);
for i=1:1:nc;
    BigEstRst(:,2*(i-1)+1) = BigBhatT(:,i);
    BigEstRst(:,2*(i-1)+2) = BigseT(:,i);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step5-0: Plot alpha_i along with index
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure1 = figure;
set(figure1, 'Units', 'inches');
set(figure1, 'Position', [1 2 2.9*3 2*3]);
set(figure1, 'PaperPositionMode', 'manual');
set(figure1, 'PaperUnits', 'inches');
set(figure1, 'PaperPosition', [1 1 2.9*3 1.6*3]);

axes1 = axes('Parent',figure1,'LineWidth',2,'Layer','top','FontSize',16,...
    'YTickLabel',{'-0.08','-0.04','0','0.04'},...
    'YTick',[-0.08 -0.04 0 0.04],...    
    'FontName','Tahoma');
ylim([-0.08 0.04]);
box(axes1,'on');
hold(axes1,'all');
stem(BigBhat(1,:),'LineWidth',1.5);
title('Fixed Time','FontSize',20,'FontName','Tahoma');                                                                                   

print(figure1,'-depsc','alpha_FF_1.eps')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step5-1: plot (betahat_mkt,i*ave(R_mkt-R_f)+betahat_sml,i*ave(R_sml)
% +betahat_hml,i*ave(R_hml) vs ave(R_i-R_f)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Pairs = [mean(MY,1)' (mean(MX(:,2:end),1)*BigBhat(2:end,:))'];
minrange=floor(min(min(Pairs))*10)/10;
maxrange=ceil(max(max(Pairs))*10)/10;

figure2 = figure;
set(figure2, 'Units', 'inches');
set(figure2, 'Position', [1 2 2.9*3 2*3]);
set(figure2, 'PaperPositionMode', 'manual');
set(figure2, 'PaperUnits', 'inches');
set(figure2, 'PaperPosition', [1 1 2.9*3 1.6*3]);

axes2 = axes('Parent',figure2,'LineWidth',2,'Layer','top','FontSize',16,...
    'FontName','Tahoma');
xlim([minrange maxrange]);ylim([minrange maxrange]);hold('all');
box(axes2,'on');
hold(axes2,'all');
scatter(Pairs(:,1),Pairs(:,2),'LineWidth',2);                                                                                
annotation(figure2,'line',[0.13 0.905],[0.11 0.925],'LineStyle',':',...
    'LineWidth',2);
print(figure2,'-depsc','beta_FF_1.eps')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step6: Estimate error variance matrix
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Sigmahat    = Uhat'*Uhat/length(Uhat);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step7: Calcuate Wald statistic
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

X1 = MX(:,1);
X2 = MX(:,2:end);

star = inv(X1'*X1-X1'*X2*inv(X2'*X2)*X2'*X1);
Wald = inv(star)*BigBhat(1,:)*inv(Sigmahat)*BigBhat(1,:)';

fprintf('\n H_0: alpha_i = 0 for all i.');
fprintf('\n Wald statistic is                   %10.4f. ', Wald);
fprintf('\n p-value from Wald statistic is      %10.4f. \n',1-chi2cdf(Wald,n));       

