% Evaluating Factor Pricing Models Using High Frequency Panels
% by Yoosoon Chang, Yongok Choi, Hagen Kim, Joon Park
%
% Average Realized Volatility for Fitted Residuals
% Input: 1. FF_Factors_daily.txt
%           1963.7.1 ~ 2009. 3.31 for the risk free rate
%        2. VW_25ptfo_daily.txt
%           1963.7.1 ~ 2008. 12.31 for the 25 portfolios(Size,B/M)
%        3. Dm_Mkt.txt
%           1963.7.2 ~ 2008. 12.31 for the demeaned excess market return
% Output: Two figures: AV_Vol_FT.eps and AV_Vol_RT.eps
%
% 2014-03-05  
% Version 2.0 
% By Yongok Choi (choiyongok@gmail.com)

clear all; close all; clc;
addpath('..\..\SharedPrg');

% Loading data sets
% 1963.7.1 ~ 2008. 12.31 for the 5x5 portfolio
% 1963.7.1 ~ 2009. 3.31 for the F-F factor
% 1963.7.2 ~ 2008. 12.31 for the demeaned excess Market return
load ('..\..\Data\VW_25ptfo_daily.txt');           % Value-weighted returns
load ('..\..\Data\FF_Factors_daily.txt');          % Fama-French factors and risk-free rate
load ('..\..\Data\Dm_Mkt.txt');                    % demeaned excess market return

% Define variables
% 25 returns, percentage terms
% Size = 1 (smallest) to 5 (largest) scales, B/M = 1 (low) to 5 (high) 
% R(i), i = 1,...,25. i=1 corresponds to (size,B/M)=(1,1), i=2,
% (1,2),..., i=5 to (1,5). Then, i=6 is (2,1) and so on. Thus, i=25 means
% (biggest firm, highest B/M). 

R_vw    = VW_25ptfo_daily(:,2:end); %Value-weighted returns
[m,n]   = size(R_vw); % m: # of rows, n: # of columns

% FF factors and risk-free rate
Mkt  = FF_Factors_daily(1:m,2);     %Market minus risk free rate
Rf   = FF_Factors_daily(1:m,end);   %Risk free rate
SMB  = FF_Factors_daily(1:m,3);     %Small minus Big ME
HML  = FF_Factors_daily(1:m,4);     %High minus Low BE/ME

% Excess 1-peiod holding period returns and factors in percentage terms
hprx_vw = R_vw(2:end,:)-kron(Rf(1:end-1,:),ones(1,n));
Mkt     = Mkt(2:end,:);
SMB     = SMB(2:end,:);
HML     = HML(2:end,:);
Dm_Mkt  = Dm_Mkt(1:end,:);

Date    = VW_25ptfo_daily(2:end,1);     %Date
Year    = floor(Date/10000);            %Year
Month   = floor(Date/100)-Year*100;     %Month
Mindex  = (Year-1963)*12+Month;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step0: Set the decade that we want to investigate
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SY      = 1963;
SM      = 7;
EY      = 2008;
EM      = 12;

DI_1    = Mindex >= (SY-1963)*12+SM;
DI_2    = Mindex <= (EY-1963)*12+EM;
DI      = DI_1 & DI_2;

hprx_vw     = hprx_vw(DI,:);
Mkt         = Mkt(DI,:);
SMB         = SMB(DI,:);
HML         = HML(DI,:);
Dm_Mkt      = Dm_Mkt(DI,:);

Date        = Date(DI,:);
Mindex      = Mindex(DI,:);

MSE_p       = 0.05;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step1: Fix at monthly average QV of excess mkt(demeaned) return for time change
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

QV      = cumsum(Dm_Mkt.^2,1);                     %Quadratic Variation
TQV     = sum(Dm_Mkt.^2,1);                        %Total Quadratic Variation
MDelta  = TQV/(max(Mindex)-min(Mindex)+1);      %Monthly Equivalent Delta

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step2: Jump Test by Lee and Mykland
%       - get the date and month information
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

K       = 16;                               %window size
alpha   = 0.05;                             %significance level

rejidx   = JT_LM(Mkt,K,alpha);              %reject index(jump indicate)
Jumpdate = Date(rejidx);                  %Date for Jumps

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step3: Select valid time changed sample
%       - remove samples whose period includes Jumps
%       - remove samples whose contribution to MSE is high
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[TCD,Tms,DeltaV] = TimeChange(Dm_Mkt,MDelta,3);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Constructing Time Change Interval Information
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Sdate   = [Date(1);Date(Tms(1:end-1)+1)];
Edate   = Date(Tms(1:end));
Nobs    = [Tms(1);Tms(2:end)-Tms(1:end-1)];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% treat Jumps
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Jump    = zeros(length(Tms),1);
for i = 1:1:length(Tms)
    if i == 1
        Jump(i) = any(rejidx(1:Tms(1)));
    else
        Jump(i) = any(rejidx(Tms(i-1)+1:Tms(i)));
    end
end

VP_Jump = Jump < 1; % Valid Periods without Jumps
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% treat MSE
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
S_SE    = sort((DeltaV-MDelta).^2,1,'descend');%Sorted Square Error

VP_MSE  = (DeltaV-MDelta).^2 < S_SE(round(length(Tms)*MSE_p));   %Valid Periods in terms of MSE 
VP      = VP_MSE & VP_Jump;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step4: Build Monthly Data 
%       - remove months which jumps happened
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

NM      = length(Nobs);
MY      = zeros(NM,n);
MX      = zeros(NM,4);

for i=1:1:NM;
    S_I_1   = Date >= Sdate(i,1);
    S_I_2   = Date <= Edate(i,1);
    S_I     = S_I_1 & S_I_2;
    Temp_Y  = hprx_vw(S_I,:);
    Temp_X  = [Mkt(S_I,:) SMB(S_I,:) HML(S_I,:)];
    MX(i,:) = [sum(S_I,1)*100/252 sum(Temp_X,1)];
    MY(i,:) = sum(Temp_Y,1);
end

MX  = MX(VP,:);
MY  = MY(VP,:);
RT_Year = floor(Edate(VP,:)/10000);
RT_Month = floor(Edate(VP,:)/100)-RT_Year*100;
RT_Time =RT_Year+RT_Month/12;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step3: For each i=1, ... , 25, estimate betas by running OLS over Monthly
% data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

BigBhat = inv(MX'*MX)*MX'*MY;
BigBhatT = BigBhat';
BetaRst =[ BigBhatT;mean(BigBhatT,1);std(BigBhatT,1) ];
Uhat    = MY - MX*BigBhat;
 
[nr,nc]     = size(MX);
s2          = sum(Uhat.^2,1)/(nr-nc);
Bigse       = sqrt(diag(inv(MX'*MX))*s2);
BigseT      = Bigse';

BigEstRst   = zeros(n,nc*2);
for i=1:1:nc;
    BigEstRst(:,2*(i-1)+1) = BigBhatT(:,i);
    BigEstRst(:,2*(i-1)+2) = BigseT(:,i);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step4: construct fitted residuals at daily frequncy using beta hat 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

BigUhat = hprx_vw - [ones(length(Mkt),1)*100/252 Mkt SMB HML]*BigBhat;

% RT Residual's QV
NM          = length(Nobs);
MRsd_RT     = zeros(nr,25);
MRsd_AR_RT  = zeros(25,2);

for j=1:1:25;
    MRsd_RT_TP = zeros(NM,1);
for i=1:1:NM;
    S_I_1   = Date >= Sdate(i,1);
    S_I_2   = Date <= Edate(i,1);
    S_I     = S_I_1 & S_I_2;
    Temp_X  = BigUhat(S_I,j);
    MRsd_RT_TP(i,1) = sum(Temp_X.^2,1);   
end

MRsd_RT(:,j)  = MRsd_RT_TP(VP,:);

Y = MRsd_RT(2:end,j);
X = [ones(length(Y),1) MRsd_RT(1:end-1,j)];

MRsd_AR_RT(j,1:2) = (inv(X'*X)*X'*Y)';
end

% FT Residual's QV

Date    = VW_25ptfo_daily(2:end,1);     %Date
Year    = floor(Date/10000);            %Year
Month   = floor(Date/100)-Year*100;     %Month
Mindex  = (Year-1963)*12+Month;

SY      = 1963;
SM      = 7;
EY      = 2008;
EM      = 12;

DI_1    = Mindex >= (SY-1963)*12+SM;
DI_2    = Mindex <= (EY-1963)*12+EM;
DI      = DI_1 & DI_2;

Dm_Mkt      = Dm_Mkt(DI,:);

Date        = Date(DI,:);
Year        = Year(DI,:);
Month       = Month(DI,:); 
Mindex      = Mindex(DI,:);


SMI     = (SY-1963)*12+SM;
EMI     = (EY-1963)*12+EM;
NM      = EMI-SMI+1;
MRsd_FT     = zeros(NM,25);
MRsd_AR_FT  = zeros(25,2);
FT_Time = (1963+7/12:1/12:2009)';
for j=1:1:25;
    MRsd_FT_TP = zeros(NM,1);
for i=1:1:NM;
    S_I     = Mindex == i+SMI-1;
    Temp_X  = BigUhat(S_I,j);
    MRsd_FT_TP(i,:) = sum(Temp_X.^2,1);
end

MRsd_FT(:,j)=MRsd_FT_TP;

Y = MRsd_FT(2:end,j);
X = [ones(length(Y),1) MRsd_FT(1:end-1,j)];

MRsd_AR_FT(j,1:2) = (inv(X'*X)*X'*Y)';
end

SMRsd_FT=sqrt(MRsd_FT);
SMRsd_RT=sqrt(MRsd_RT);

AV_SMRsd_FT=mean(SMRsd_FT,2);
AV_SMRsd_RT=mean(SMRsd_RT,2);

figure1 = figure;
set(figure1, 'Units', 'inches');
set(figure1, 'Position', [1 2 2.9*3 2*3]);
set(figure1, 'PaperPositionMode', 'manual');
set(figure1, 'PaperUnits', 'inches');
set(figure1, 'PaperPosition', [1 1 2.9*3 2*3]);

axes1 = axes('Parent',figure1,'LineWidth',2,'Layer','top','FontSize',16,...
    'YTickLabel',{'0','2','4','6'},...
    'YTick',[0 2 4 6],...
    'FontName','Tahoma');
ylim([0 6]);
xlim([1965 2010]);hold('all');
box(axes1,'on');
hold(axes1,'all');
plot(FT_Time,AV_SMRsd_FT,'Color', 'b','LineWidth',2);
                                                                          
title('Fixed Time','FontSize',20,'FontName','Tahoma');  

print(figure1,'-depsc','AV_Vol_FT.eps')

figure2 = figure;
set(figure2, 'Units', 'inches');
set(figure2, 'Position', [1 2 2.9*3 2*3]);
set(figure2, 'PaperPositionMode', 'manual');
set(figure2, 'PaperUnits', 'inches');
set(figure2, 'PaperPosition', [1 1 2.9*3 2*3]);

axes1 = axes('Parent',figure2,'LineWidth',2,'Layer','top','FontSize',16,...
    'YTickLabel',{'0','2','4','6'},...
    'YTick',[0 2 4 6],...
    'FontName','Tahoma');
ylim([0 6]);
xlim([1965 2010]);hold('all');
box(axes1,'on');
hold(axes1,'all');
plot(RT_Time,AV_SMRsd_RT,'Color', 'r','LineWidth',2);                                                                              
title('Random Time','FontSize',20,'FontName','Tahoma'); 

print(figure2,'-depsc','AV_Vol_RT.eps')
