% Evaluating Factor Pricing Models Using High Frequency Panels
% by Yoosoon Chang, Yongok Choi, Hagen Kim, Joon Park
%
% Realized Volatility from Excess Market Return
% Input: 1. VW_25ptfo_daily.txt
%           1963.7.1 ~ 2008. 12.31 for the 25 portfolios(Size,B/M)
%        2. Dm_Mkt.txt
%           1963.7.2 ~ 2008. 12.31 for the demeaned excess market return
% Output: Two figures: Mkt_RVol_FT.eps Mkt_RVol_RT.eps
%
% 2014-03-05  
% Version 2.0 
% By Yongok Choi (choiyongok@gmail.com)

clear all; close all; clc;
addpath('..\..\SharedPrg');

% Loading data sets
% 1963.7.1 ~ 2008. 12.31 for the 5x5 portfolio
% 1963.7.2 ~ 2008. 12.31 for the demeaned excess Market return
load ('..\..\Data\VW_25ptfo_daily.txt');               %Value-weighted returns
load ('..\..\Data\Dm_Mkt.txt');                        %Demeaned excess market return

Date    = VW_25ptfo_daily(2:end,1);     %Date
Year    = floor(Date/10000);            %Year
Month   = floor(Date/100)-Year*100;     %Month
Mindex  = (Year-1963)*12+Month;

SY      = 1963;
SM      = 7;
EY      = 2008;
EM      = 12;

DI_1    = Mindex >= (SY-1963)*12+SM;
DI_2    = Mindex <= (EY-1963)*12+EM;
DI      = DI_1 & DI_2;

Dm_Mkt      = Dm_Mkt(DI,:);

Mindex      = Mindex(DI,:);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step1: Jump Test by Lee and Mykland
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

K       = 16;                               %window size
alpha   = 0.05;                             %significance level

rejidx   = JT_LM(Dm_Mkt,K,alpha);              %reject index(jump indicate)
% Jumpdate = Date(rejidx);                  %Date for Jumps

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step2: Build Monthly Data 
%       - remove months which jumps happened
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SMI     = (SY-1963)*12+SM;
EMI     = (EY-1963)*12+EM;
NM      = EMI-SMI+1;
MRtn_FT = zeros(NM,1);

VP      = true(NM,1);
for i=1:1:NM;
    S_I     = Mindex == i+SMI-1;
    Temp_X  = Dm_Mkt(S_I,:);
    MRtn_FT(i,:) = sum(Temp_X.^2,1);
    VP(i)   = sum(rejidx(S_I),1) <10 ;    
end
FT_Time = (1963+7/12:1/12:2009)';
MRtn_FT=sqrt(MRtn_FT(VP,1));

Y = (MRtn_FT(2:end,1));
X = [ones(length(Y),1) MRtn_FT(1:end-1,1)];

beta_FT = inv(X'*X)*X'*Y;

% Random Time
% Define variables
% 25 returns, percentage terms
% Size = 1 (smallest) to 5 (largest) scales, B/M = 1 (low) to 5 (high) 
% R(i), i = 1,...,25. i=1 corresponds to (size,B/M)=(1,1), i=2,
% (1,2),..., i=5 to (1,5). Then, i=6 is (2,1) and so on. Thus, i=25 means
% (biggest firm, highest B/M). 

R_vw    = VW_25ptfo_daily(:,2:end);                 % Value-weighted returns
[m,n]   = size(R_vw);                               % m: # of rows, n: # of columns

Dm_Mkt  = Dm_Mkt(1:end,:);

Date    = VW_25ptfo_daily(2:end,1);                 % Date
Year    = floor(Date/10000);                        % Year
Month   = floor(Date/100)-Year*100;                 % Month
Mindex  = (Year-1963)*12+Month;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step0: Set the decade that we want to investigate
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SY      = 1963;
SM      = 7;
EY      = 2008;
EM      = 12;

DI_1    = Mindex >= (SY-1963)*12+SM;
DI_2    = Mindex <= (EY-1963)*12+EM;
DI      = DI_1 & DI_2;

Dm_Mkt      = Dm_Mkt(DI,:);

Date        = Date(DI,:);
Year        = Year(DI,:);
Month       = Month(DI,:); 
Mindex      = Mindex(DI,:);

MSE_p       = 0.05;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step1: Fix at monthly average QV of excess mkt return for time change
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

QV      = cumsum(Dm_Mkt.^2,1);                      % Quadratic Variation
TQV     = sum(Dm_Mkt.^2,1);                         % Total Quadratic Variation
MDelta  = TQV/(max(Mindex)-min(Mindex)+1);          % Monthly Equivalent Delta

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step2: Jump Test by Lee and Mykland
%       - get the date and month information
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

K       = 16;                                       % window size
alpha   = 0.05;                                     % significance level

rejidx   = JT_LM(Dm_Mkt,K,alpha);                   % reject index(jump indicate)
% Jumpdate = Date(rejidx);                          % Date for Jumps

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step3: Select valid time changed sample
%       - remove samples whose period includes Jumps
%       - remove samples whose contribution to MSE is high
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[TCD,Tms,DeltaV] = TimeChange(Dm_Mkt,MDelta,3);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Constructing Time Change Interval Information
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Sdate   = [Date(1);Date(Tms(1:end-1)+1)];
Edate   = Date(Tms(1:end));
Nobs    = [Tms(1);Tms(2:end)-Tms(1:end-1)];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% treat Jumps
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Jump    = zeros(length(Tms),1);
for i = 1:1:length(Tms)
    if i == 1
        Jump(i) = any(rejidx(1:Tms(1)));
    else
        Jump(i) = any(rejidx(Tms(i-1)+1:Tms(i)));
    end
end

VP_Jump = Jump < 1; % Valid Periods without Jumps
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% treat MSE
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
S_SE    = sort((DeltaV-MDelta).^2,1,'descend');%Sorted Square Error

VP_MSE  = (DeltaV-MDelta).^2 < S_SE(round(length(Tms)*MSE_p));   %Valid Periods in terms of MSE 
VP      = VP_MSE & VP_Jump;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step4: Build Monthly Data 
%       - remove months which jumps happened
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

NM          = length(Nobs);
MRtn_RT     = zeros(NM,1);

for i=1:1:NM;
    S_I_1   = Date >= Sdate(i,1);
    S_I_2   = Date <= Edate(i,1);
    S_I     = S_I_1 & S_I_2;
    Temp_X  = Dm_Mkt(S_I,:);
    MRtn_RT(i,:) = sum(Temp_X.^2,1);   
end

RT_Year = floor(Edate(VP,:)/10000);
RT_Month = floor(Edate(VP,:)/100)-RT_Year*100;
RT_Time =RT_Year+RT_Month/12;

MRtn_RT  = sqrt(MRtn_RT(VP,:));

Y = MRtn_RT(2:end,1);
X = [ones(length(Y),1) MRtn_RT(1:end-1,1)];

beta_RT = inv(X'*X)*X'*Y;

figure1 = figure;
set(figure1, 'Units', 'inches');
set(figure1, 'Position', [1 2 2.9*3 2*3]);
set(figure1, 'PaperPositionMode', 'manual');
set(figure1, 'PaperUnits', 'inches');
set(figure1, 'PaperPosition', [1 1 2.9*3 2*3]);

axes1 = axes('Parent',figure1,'LineWidth',2,'Layer','top','FontSize',16,...
    'YTickLabel',{'0','5','10','15','20','25'},...
    'YTick',[0 5 10 15 20 25],...
    'FontName','Tahoma');
ylim([0 25]);
xlim([1965 2010]);hold('all');
box(axes1,'on');
hold(axes1,'all');
plot(FT_Time,MRtn_FT,'Color', 'b','LineWidth',2);
                                                                          
title('Fixed Time','FontSize',20,'FontName','Tahoma');  

print(figure1,'-depsc','Mkt_RVol_FT.eps')

figure2 = figure;
set(figure2, 'Units', 'inches');
set(figure2, 'Position', [1 2 2.9*3 2*3]);
set(figure2, 'PaperPositionMode', 'manual');
set(figure2, 'PaperUnits', 'inches');
set(figure2, 'PaperPosition', [1 1 2.9*3 2*3]);

axes1 = axes('Parent',figure2,'LineWidth',2,'Layer','top','FontSize',16,...
    'YTickLabel',{'0','5','10','15','20','25'},...
    'YTick',[0 5 10 15 20 25],...
    'FontName','Tahoma');
ylim([0 25]);
xlim([1965 2010]);hold('all');
box(axes1,'on');
hold(axes1,'all');
plot(RT_Time,MRtn_RT,'Color', 'r','LineWidth',2);                                                                              
title('Random Time','FontSize',20,'FontName','Tahoma'); 

print(figure2,'-depsc','Mkt_RVol_RT.eps')
