% Evaluating Factor Pricing Models Using High Frequency Panels
% by Yoosoon Chang, Yongok Choi, Hagen Kim, Joon Park
%
% Correlation between Residuals and Regressors in Random Time Regression
% Input: 1. FF_Factors_daily.txt
%           1963.7.1 ~ 2009. 3.31 for the risk free rate
%        2. VW_25ptfo_daily.txt
%           1963.7.1 ~ 2008. 12.31 for the 25 portfolios(Size,B/M)
%        3. Dm_Mkt.txt
%           1963.7.1 ~ 2008. 12.31 for the demeaned excess market return
% Output: Table_3 
%
% 2014-01-31  
% Version 2.0 
% By Yongok Choi (choiyongok@gmail.com)

% Loading data sets
% 1963.7.1 ~ 2008. 12.31 for the 5x5 portfolio
% 1963.7.1 ~ 2009. 3.31 for the F-F factor
clear all; close all; clc;
addpath('..\..\SharedPrg');

load ('..\..\Data\VW_25ptfo_daily.txt');   % Value-weighted returns
load ('..\..\Data\FF_Factors_daily.txt');  % Fama-French factors and risk-free rate
load ('..\..\Data\Dm_Mkt.txt');            % Demeaned excess market return

% Define variables
% 25 returns, percentage terms
% Size = 1 (smallest) to 5 (largest) scales, B/M = 1 (low) to 5 (high) 
% R(i), i = 1,...,25. i=1 corresponds to (size,B/M)=(1,1), i=2,
% (1,2),..., i=5 to (1,5). Then, i=6 is (2,1) and so on. Thus, i=25 means
% (biggest firm, highest B/M). 

R_vw    = VW_25ptfo_daily(:,2:end); %Value-weighted returns
[n,m] = size(R_vw); % n: # of rows, m: # of columns

% FF factors and risk-free rate
Mkt  = FF_Factors_daily(1:n,2);     %Market minus risk free rate
Rf   = FF_Factors_daily(1:n,end);   %Risk free rate
SMB  = FF_Factors_daily(1:n,3);     %Small minus Big ME
HML  = FF_Factors_daily(1:n,4);     %High minus Low BE/ME

% Excess 1-peiod holding period returns and factors in percentage terms
hprx_vw = R_vw(2:end,:)-kron(Rf(1:end-1,:),ones(1,m));
Mkt     = Mkt(2:end,:);
SMB     = SMB(2:end,:);
HML     = HML(2:end,:);
Dm_Mkt  = Dm_Mkt(1:end,:);

Date    = VW_25ptfo_daily(2:end,1);     %Date
Year    = floor(Date/10000);            %Year
Month   = floor(Date/100)-Year*100;     %Month
Mindex  = (Year-1963)*12+Month;         %Monthly Index

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step1: Jump Test by Lee and Mykland
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

K       = 16;                               %window size
alpha   = 0.05;                             %significance level

rejidx   = JT_LM(Dm_Mkt,K,alpha);              %reject index(jump indicate)
Jumpdate = Date(rejidx);                    %Date for Jumps

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step2: Build Monthly Data (Physical Time)
%       - remove months which jumps happened
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SMI     = min(Mindex);                      %Start Monthly Index
EMI     = max(Mindex);                      %End   Monthly Index
NM      = EMI-SMI+1;                        %the Number of Month
MY      = zeros(NM,m);                      %Monthly Y
MX      = zeros(NM,4);                      %Monthly X
VP      = true(NM,1);                       %Valid Period Index

for i=1:1:NM;
    S_I     = Mindex == i+SMI-1;
    Temp_Y  = hprx_vw(S_I,:);
    Temp_X  = [Mkt(S_I,:) SMB(S_I,:) HML(S_I,:)];
    MX(i,:) = [sum(S_I,1) sum(Temp_X,1)];
    MY(i,:) = sum(Temp_Y,1);
    VP(i)   = sum(rejidx(S_I),1) == 0 ;    
end

MX  = MX(VP,:);
MY  = MY(VP,:);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step3: For each i=1, ... , 25, estimate betas by running OLS over Monthly
% data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

BigBhat = inv(MX'*MX)*MX'*MY;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step4: construct fitted residuals at daily frequncy using beta hat 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

BigUhat = hprx_vw - [ones(length(Mkt),1) Mkt SMB HML]*BigBhat;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step5: Fix at monthly average QV of excess mkt return for time change
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

QV      = cumsum(Dm_Mkt.^2,1);                   %Quadratic Variation
TQV     = sum(Dm_Mkt.^2,1);                      %Total Quadratic Variation
MDelta  = TQV/(max(Mindex)-min(Mindex)+1);       %Monthly Equivalent Delta

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step6: Select valid time changed sample
%       - remove samples whose period includes Jumps
%       - remove samples whose contribution to MSE is high
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[TCD,Tms,DeltaV] = TimeChange(Dm_Mkt,MDelta,3);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Constructing Time Change Interval Information
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Sdate   = [Date(1);Date(Tms(1:end-1)+1)];   %Start Date
Edate   = Date(Tms(1:end));                 %End Date
Nobs    = [Tms(1);Tms(2:end)-Tms(1:end-1)]; %the number of observations for each period

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% treat Jumps
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Jump    = zeros(length(Tms),1);
for i = 1:1:length(Tms)
    if i == 1
        Jump(i) = any(rejidx(1:Tms(1)));
    else
        Jump(i) = any(rejidx(Tms(i-1)+1:Tms(i)));
    end
end

VP_Jump = Jump < 1; % Valid Periods without Jumps

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% treat MSE
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
S_SE    = sort((DeltaV-MDelta).^2,1,'descend');%Sorted Square Error
MSE_plot= zeros(length(Tms),1);

VP_MSE  = (DeltaV-MDelta).^2 < S_SE(round(length(Tms)*0.05));   %Valid Periods in terms of MSE 
VP      = VP_MSE & VP_Jump;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step7: Build Time changed Monthly Data 
%       - remove samples which jumps happened
%       - remove samples violating MSE criterion
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

NM      = length(Nobs);
MU      = zeros(NM,m);
MX      = zeros(NM,4);

for i=1:1:NM;
    S_I_1   = Date >= Sdate(i,1);
    S_I_2   = Date <= Edate(i,1);
    S_I     = S_I_1 & S_I_2;
    Temp_U  = BigUhat(S_I,:);
    Temp_X  = [Mkt(S_I,:) SMB(S_I,:) HML(S_I,:)];
    MX(i,:) = [sum(S_I,1) sum(Temp_X,1)];
    MU(i,:) = sum(Temp_U,1);
end

TCX = MX(VP,:);
TCU = MU(VP,:);

Corr1   = corr(TCU,TCX);

Table4_Rst  = [Corr1 abs(Corr1);mean(Corr1) mean(abs(Corr1));std(Corr1) std(abs(Corr1))];
