function llik = Lik_lsv(para,y1)
% Evaluate log-likelihood ftn.
% In case of nonstationary process, we cannot identify x_0 and kappa at the
% same time.
% Here we set x_0 = 0, so that we can use this function for the
% nonstationary case as well as for the stationary case.
% For the stationary, we don't have to set x_0 = 0, but here we just assume
% x_0 = 0.
%
%   2012-12-02  
%   Version 2.0 
%   By Yongok Choi (choiyongok@gmail.com)

% Parameter re-transformation
theta1  = Trns_LSV(para);

% parameters
mu      = theta1(1);
nu      = theta1(2);
lambda  = theta1(3);
gamma   = theta1(4);
alpha   = theta1(5);
rho     = theta1(6);
  
% initial values
% dx: tic size for integral wrt x
% Dx: Domain of x
% f_0: f(x_0)
% py_tl: {p(y_0), p(y_1|F_0), ... , p(y_{T-1}|F_{T-2})}
% dv: tic size for integral wrt v
% v: Domain of v

dx      = 0.4;
Dx      = (-20:dx:20)';                            
x_0     = 0;                                        
f_0     = mu + nu/(1+exp(-lambda*0+gamma));       
y_0     = y1(1,1);
py_tl   = zeros(length(y1),1);                          
py_tl(1,1) = 1/sqrt(2*pi)*exp(-1/2*(y_0/f_0)^2); % p(y_0), y_0 ~ N(0, f_0^2)

dv      = 0.2;
v       =(-3:dv:3)';

% t = 1
% prediction step
% x_tl: x_{1|0}
% px_tl: p(x_1|F_0), x_1|F_0 ~ N(alpha*x_0 + rho/f(x_0)*y_0, 1-rho^2)
x_tl    = alpha*x_0+rho/f_0*y_0;
px_tl   = exp(-1/(2*(1-rho^2)).*(Dx+x_tl-rho*y_0/f_0).^2)/sqrt(2*pi*(1-rho^2))*dx;

% updating step
% f_t: f(x) for x=x_1
% pxy_ttl: p(x_1,y_1|F_0), p(x_1,y_1|F_0) = p(y_1|x_1)*p(x_1|F_0)
%          y_1|x_1 ~ N(0,f(x_1))
% py_tl(2,1): p(y_1|F_0)
% px_tt: p(x_1|F_1) 
% x_tt: x_{1|1}
f_t     = mu + nu./(1+exp(-lambda.*(Dx+x_tl)+gamma));
pxy_ttl = px_tl.*(1./sqrt(2*pi*f_t.^2).*exp(-y1(2,1)^2./(2.*f_t.^2)));
py_tl(2,1)   = sum(pxy_ttl)*dx;
px_tt   = pxy_ttl/py_tl(2,1);
x_tt    = sum(Dx.*px_tt)*dx+x_tl;

% t=2:T-1
for j=2:length(y1)-1
    % Prediction
    % pv_tl: p(v_t|F_t)
    % x_ldl: x_{t-1|t-2}
    % x_tl: x_{t|t-1}
    % px_tl: p(x_t|F_{t-1}) around x_{t|t-1}   
    pv_tl   = dx/sqrt(2*pi*(1-rho^2))*(exp((-(kron(v,ones(1,length(px_tt))) ...
                -kron(ones(length(v),1),(rho*y1(j,1)./f_t)')).^2)/(2*(1-rho^2))))* px_tt;
    x_ldl   = x_tl;
    x_tl    = alpha*x_tt + sum(v.*pv_tl)*dv;
    px_tl   = dx/sqrt(2*pi*(1-rho^2))*(exp((-(kron(Dx+x_tl,ones(1,length(px_tt))) ...
                -kron(ones(length(Dx),1),(alpha*(Dx+x_ldl)+rho*y1(j,1)./f_t)')).^2)/(2*(1-rho^2))))* px_tt;
    % Updating
    % f_t: f(x) for x=x_t
    % pxy_ttl: p(x_t,y_t|F_{t-1}), p(x_t,y_t|F_{t-1}) = p(y_t|x_t)*p(x_t|F_{t-1})
    %          y_t|x_t ~ N(0,f(x_t)^2)
    % py_tl(t+1,1): p(y_t|F_{t-1}) 
    % px_tt: p(x_t|F_t) around x_{t|t-1}
    % x_tt: x_{t|t}    
    f_t     = mu + nu./(1+exp(-lambda.*(Dx+x_tl)+gamma));
    pxy_ttl = px_tl.*(1./sqrt(2*pi*f_t.^2).*exp(-y1(j+1,1)^2./(2.*f_t.^2)));
    py_tl(j+1,1) = sum(pxy_ttl)*dx;
    px_tt   = pxy_ttl/py_tl(j+1,1);
    x_tt    = sum(Dx.*px_tt)*dx+x_tl;
end

%----------------------------------------------------------
% log-likelihood: llik
%----------------------------------------------------------
  llik = - sum(log(py_tl));
  
fprintf('para=[%3.5f, %3.5f, %3.5f, %3.5f, %3.5f, %3.5f] lik = %3.5f \n', ...
         mu, nu, lambda, gamma, alpha, rho, llik);  
  
  
return;
