% Evaluating Factor Pricing Models Using High Frequency Panels
% by Yoosoon Chang, Yongok Choi, Hagen Kim, Joon Park
%
% CAPM on Book-to-Market Portfolio
% Input: 1. FF_Factors_daily.txt
%           1963.7.1 ~ 2009. 3.31 for the risk free rate
%        2. 11_portfolios_BE_ME.txt
%           1963.7.1 ~ 2008. 12.31 for the 11 portfolios(B/M)
%        3. Dm_Mkt.txt
%           1963.7.1 ~ 2008. 12.31 for the demeaned excess market return
% Output: Table_5_Left 
%
% 2014-02-05  
% Version 2.0 
% By Yongok Choi (choiyongok@gmail.com)
%
% Loading data sets
% 1963.7.1 ~ 2008. 12.31 for the 11 portfolio
% 1963.7.1 ~ 2009. 3.31 for the F-F factor
clear all; close all; clc;
addpath('..\..\SharedPrg');

load ('..\..\Data\11_portfolios_BE_ME.txt');           %Value-weighted returns
load ('..\..\Data\FF_Factors_daily.txt');              %Fama-French factors and risk-free rate
load ('..\..\Data\Dm_Mkt.txt');                        %Demeaned excess market return

% Define variables
% Value Weigted Returns (daily, percentage)											
% 10 portfolios based on BE_ME ratio plus the Hi-Lo portfolio (11th)
% Note that 11th regression is redundant, so we run 10 regressions

R_vw    = X11_portfolios_BE_ME(:,2:end-1); %Value-weighted returns
[m,n]   = size(R_vw); % m: # of rows, n: # of columns

% FF factors and risk-free rate
Mkt  = FF_Factors_daily(1:m,2);     %Market minus risk free rate
Rf   = FF_Factors_daily(1:m,end);   %Risk free rate

% Excess 1-peiod holding period returns and factors in percentage terms
hprx_vw = R_vw(2:end,:)-kron(Rf(1:end-1,:),ones(1,n));
Mkt     = Mkt(2:end,:);
Dm_Mkt  = Dm_Mkt(1:end,:);

Date    = X11_portfolios_BE_ME(2:end,1);    %Date
Year    = floor(Date/10000);                %Year
Month   = floor(Date/100)-Year*100;         %Month
Mindex  = (Year-1963)*12+Month;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step0: Set the decade that we want to investigate
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SY      = 1963;
SM      = 7;
EY      = 2008;
EM      = 12;

DI_1    = Mindex >= (SY-1963)*12+SM;
DI_2    = Mindex <= (EY-1963)*12+EM;
DI      = DI_1 & DI_2;

hprx_vw     = hprx_vw(DI,:);
Mkt         = Mkt(DI,:);
Dm_Mkt      = Dm_Mkt(DI,:);

Date        = Date(DI,:);
Year        = Year(DI,:);
Month       = Month(DI,:); 
Mindex      = Mindex(DI,:);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step1: Jump Test by Lee and Mykland
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

K       = 16;                               %window size
alpha   = 0.05;                             %significance level

rejidx   = JT_LM(Dm_Mkt,K,alpha);              %reject index(jump indicate)
% Jumpdate = Date(rejidx);                  %Date for Jumps

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step2: Build Monthly Data 
%       - remove months which jumps happened
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SMI     = (SY-1963)*12+SM;
EMI     = (EY-1963)*12+EM;
NM      = EMI-SMI+1;
MY      = zeros(NM,n);
MX      = zeros(NM,2);
MU      = zeros(NM,n);
VP      = true(NM,1);
for i=1:1:NM;
    S_I     = Mindex == i+SMI-1;
    Temp_Y  = hprx_vw(S_I,:);
    Temp_X  = Mkt(S_I,:);
    MX(i,:) = [sum(S_I,1)/252*100 sum(Temp_X,1)];
    MY(i,:) = sum(Temp_Y,1);
    VP(i)   = sum(rejidx(S_I),1) == 0 ;    
end

MX  = MX(VP,:);
MY  = MY(VP,:);

fprintf('Data period: from %6.0f - %2.0f to %6.0f - %2.0f',SY,SM,EY,EM);
fprintf('\n the number of the months is         %10.4f. ', NM);
fprintf('\n The number of samples with Jumps is %10.4f. ', NM-sum(VP));
fprintf('\n The number of used samples is       %10.4f. ', sum(VP));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step3: For each i=1, ... , 10, estimate betas by running OLS over Monthly
% data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

BigBhat = inv(MX'*MX)*MX'*MY;
BigBhatT = BigBhat';
Uhat    = MY - MX*BigBhat;

[nr,nc]     = size(MX);
s2          = sum(Uhat.^2,1)/(nr-nc);
Bigse       = sqrt(diag(inv(MX'*MX))*s2);
BigseT      = Bigse';

TY      = MY(:,10)-MY(:,1);
Te      = TY-MX*inv(MX'*MX)*MX'*TY;
STs2    = Te'*Te/(nr-nc);

STse    = sqrt(diag(inv(MX'*MX))*STs2);
fprintf('\n standard errors of the strategy is %10.4f',STse);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step5: Estimate error variance matrix
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Sigmahat    = Uhat(:,1:n)'*Uhat(:,1:n)/length(Uhat);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step6: Calcuate Wald statistic
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

X1 = MX(:,1);
X2 = MX(:,2);

star = inv(X1'*X1-X1'*X2*inv(X2'*X2)*X2'*X1);
Wald = inv(star)*BigBhat(1,:)*inv(Sigmahat)*BigBhat(1,:)';

fprintf('\n H_0: alpha_i = 0 for all i.');
fprintf('\n Wald statistic is                   %10.4f. ', Wald);
fprintf('\n p-value from Wald statistic is      %10.4f. \n',1-chi2cdf(Wald,n));       

Table7_Left             = zeros(23,2);
Table7_Left(1:2:20,:)   = BigBhatT;
Table7_Left(2:2:20,:)   = BigseT;
Table7_Left(21,:)       = (inv(MX'*MX)*MX'*TY)';
Table7_Left(22,:)       = (sqrt(diag(inv(MX'*MX))*STs2))';
Table7_Left(23,1)       = Wald;
Table7_Left(23,2)       = 1-chi2cdf(Wald,n);
